//===================================================================
//  DEFEDIT.C - An enhancment to the Shell functions FindExecutable
//  and ShellExecute
//  (c) Douglas Boling 1995
//
//  For better readability, set tab stops to every 3 characters.
//===================================================================
#include <windows.h>
#include "e:\projects\defedit\defedit.h"
//
// Standard procedure instance used to pass on call to old DLL.  This
// code assumes a standard Windows DLL stack frame and the pushing
// of DI and SI on the stack before the code is reached. 
//
// Note the differences between the Borland C and Microsoft C macros.
// Aside from syntax differences, BC pushes DI and SI on the stack
// in a different order than does MSC.
//
#ifdef __BORLANDC__
#define STDPROC(fnptr) FARPROC lpfn##fnptr;\
void CALLBACK fnptr (void) {\
	asm {\
		mov 	ax,word ptr lpfn##fnptr;		\
		mov 	ds,word ptr lpfn##fnptr+2;		\
		xchg	ax,ss:[bp-2];						\
		push	ax;									\
		mov 	ax,ds;								\
		xchg	ax,ss:[bp];							\
		dec 	ax;									\
		mov 	bp,ax;								\
		pop 	ds;									\
		pop 	di;									\
		pop 	si;									\
		ret											\
	}\
}
#else
#define STDPROC(fnptr) FARPROC lpfn##fnptr;\
void CALLBACK fnptr (void) {\
	_asm {\
		_asm mov 	ax,word ptr lpfn##fnptr		\
		_asm mov 	ds,word ptr lpfn##fnptr+2	\
		_asm xchg	ax,ss:[bp-2]					\
		_asm push	ax									\
		_asm mov 	ax,ds								\
		_asm xchg	ax,ss:[bp]						\
		_asm dec 	ax									\
		_asm mov 	bp,ax								\
		_asm pop 	ds									\
		_asm pop 	si									\
		_asm pop 	di									\
		_asm retf										\
	}\
}
#endif

//
// Structure used to hold Ordinal information
//
typedef struct {
	char *pszOrdName;
	FARPROC *plpfn;
} ORDSTRUCT;	

//
// Ptrs to DLL routines that are taken over by this DLL
//
typedef HINSTANCE (CALLBACK FEPROC) (LPCSTR, LPCSTR, LPSTR);
typedef FEPROC FAR *LPFEPROC;
LPFEPROC lpfnFINDEXECUTABLE;

typedef HINSTANCE (CALLBACK SEPROC) (HWND, LPCSTR, LPCSTR, LPCSTR, 
                                     LPCSTR, int);
typedef SEPROC FAR *LPSEPROC;
LPSEPROC lpfnSHELLEXECUTE;

//
// Generic procedures to pass exports though to old DLL
//
STDPROC (REGOPENKEY)
STDPROC (REGCREATEKEY)
STDPROC (REGCLOSEKEY)
STDPROC (REGDELETEKEY)
STDPROC (REGSETVALUE)
STDPROC (REGQUERYVALUE)
STDPROC (REGENUMKEY)
STDPROC (DRAGACCEPTFILES)
STDPROC (DRAGQUERYFILE)
STDPROC (DRAGFINISH)
STDPROC (DRAGQUERYPOINT)
STDPROC (SHELLABOUT)
STDPROC (WCI)
STDPROC (ABOUTDLGPROC)
STDPROC (EXTRACTICON)
STDPROC (EXTRACTASSOCIATEDICON)
STDPROC (DOENVIRONMENTSUBST)
STDPROC (FINDENVIRONMENTSTRING)
STDPROC (INTERNALEXTRACTICON)
STDPROC (HERETHARBETYGARS)
STDPROC (FINDEXEDLGPROC)
STDPROC (REGISTERSHELLHOOK)
STDPROC (SHELLHOOKPROC)

HANDLE hInst, hInstOldDLL;
char szOldLib[] = "OLDSHELL.DLL";
//
// Array that contains the names of the ordinals of the old DLL
//
ORDSTRUCT OrdArray[] = {
	{"REGOPENKEY", &lpfnREGOPENKEY},
	{"REGCREATEKEY", &lpfnREGCREATEKEY},
	{"REGCLOSEKEY", &lpfnREGCLOSEKEY},
	{"REGDELETEKEY", &lpfnREGDELETEKEY},
	{"REGSETVALUE", &lpfnREGSETVALUE},
	{"REGQUERYVALUE", &lpfnREGQUERYVALUE},
	{"REGENUMKEY", &lpfnREGENUMKEY},
	{"DRAGACCEPTFILES", &lpfnDRAGACCEPTFILES},
	{"DRAGQUERYFILE", &lpfnDRAGQUERYFILE},
	{"DRAGFINISH", &lpfnDRAGFINISH},
	{"DRAGQUERYPOINT", &lpfnDRAGQUERYPOINT},
	{"SHELLEXECUTE", &lpfnSHELLEXECUTE},
	{"FINDEXECUTABLE", &lpfnFINDEXECUTABLE},
	{"SHELLABOUT", &lpfnSHELLABOUT},
	{"WCI", &lpfnWCI},
	{"ABOUTDLGPROC", &lpfnABOUTDLGPROC},
	{"EXTRACTICON", &lpfnEXTRACTICON},
	{"EXTRACTASSOCIATEDICON", &lpfnEXTRACTASSOCIATEDICON},
	{"DOENVIRONMENTSUBST", &lpfnDOENVIRONMENTSUBST},
	{"FINDENVIRONMENTSTRING", &lpfnFINDENVIRONMENTSTRING},
	{"INTERNALEXTRACTICON", &lpfnINTERNALEXTRACTICON},
	{"HERETHARBETYGARS", &lpfnHERETHARBETYGARS},
	{"FINDEXEDLGPROC", &lpfnFINDEXEDLGPROC},
	{"REGISTERSHELLHOOK", &lpfnREGISTERSHELLHOOK},
	{"SHELLHOOKPROC", &lpfnSHELLHOOKPROC}
};

char szTemp[256];
char szDefDir[256];
char szDefEditor[128] = "";

void GetEditString (LPSTR, INT);
void ParseEditCmd (LPSTR, LPSTR, LPSTR, LPSTR);
void MyChDrive (char);
int MyChDir (LPSTR);
//===================================================================
// Library initialization and terminataion functions
//===================================================================
//-------------------------------------------------------------------
// LibMain - DLL initialization routine
// 
// The main task of this routine is to get and save the addresses
// of the exported entry points of the DLL we are trying to wrap.
//-------------------------------------------------------------------
int CALLBACK LibMain (HANDLE hInstance, WORD wDataSeg, 
                      WORD wHeapSize, LPSTR lpszCmdLine) {
	INT i;

	hInst = hInstance;
	hInstOldDLL = LoadLibrary (szOldLib);
	if (hInstOldDLL == 0)
		return 0;

	for (i = 0; i < dim (OrdArray); i++) {
		*(OrdArray[i].plpfn) = GetProcAddress (hInstOldDLL, 
		                                       OrdArray[i].pszOrdName);
		if (*(OrdArray[i].plpfn) == 0) {
			OutputDebugString ("Could not access all proc addresses\n");
			FreeLibrary (hInstOldDLL);
			return 0;
		}
	}
	// Get the default editor
	GetEditString (szDefEditor, sizeof (szDefEditor));
	return 1;
}
//-------------------------------------------------------------------
// WEP - DLL termination routine
//-------------------------------------------------------------------
int CALLBACK WEP (int nParameter) {
   
	FreeLibrary (hInstOldDLL);
	return 1;
}
//===================================================================
// Revised library exported functions
//===================================================================
//-------------------------------------------------------------------
// FindExecutable - Locates editor for a file extension
//-------------------------------------------------------------------
HINSTANCE CALLBACK FindExecutable (LPSTR lpszFile, LPSTR lpszDir, 
                                   LPSTR lpszResult) {
	HINSTANCE hInstRC;

//LPSTR lpszTest;
//OutputDebugString ("File name:->");
//OutputDebugString (lpszFile);
//OutputDebugString ("<--\n");
//OutputDebugString ("Dir name:->");
//OutputDebugString (lpszDir);
//OutputDebugString ("<--\n");


	if (lpszResult == 0) {
		GetEditString (szDefEditor, sizeof (szDefEditor));
		hInstRC = 0;
	} else {									 
		hInstRC = (lpfnFINDEXECUTABLE) (lpszFile, lpszDir, lpszResult);
		
		if ((hInstRC == 31) || (hInstRC == 2)) {
			ParseEditCmd (szDefEditor, lpszFile, lpszDir, lpszResult);

//OutputDebugString ("Before truncation:->");
//OutputDebugString (lpszResult);
//OutputDebugString ("<--\n");
//lpszTest = lpszResult;
	
			// Now that we have nicly appended the parsed filename
			// truncate it to conform to FindExecutable Spec.
			while ((*lpszResult != '\0') && (*lpszResult > ' '))
				lpszResult++;
			*lpszResult = '\0';	

//OutputDebugString ("After truncation:-->");
//OutputDebugString (lpszTest);
//OutputDebugString ("<--\n");


			// Use a return code different from the 1000 which is
			// normally used by Shell.
			hInstRC = 33;
		}		
	}	
	return hInstRC;
}
//-------------------------------------------------------------------
// ShellExecute - Locates and launches an editor for a file given
// its extension. 
//-------------------------------------------------------------------
HINSTANCE CALLBACK ShellExecute (HWND hWnd, LPSTR lpszOp, LPSTR lpszFile, 
                                 LPSTR lpszParams, LPSTR lpszDir, 
                                 int fsShowCmd) {
	HINSTANCE hInstRC;

	// Find editor for file
	hInstRC = FindExecutable (lpszFile, lpszDir, szTemp);

	// If return == 33, it is our default editor.  If real association
	// exists, pass call on to old ShellExecute.
	if (hInstRC == 33) {
		
		// Change to default directory
		if (lpszDir) {
			lstrcpy (szDefDir, lpszDir);
			if (szDefDir[1] == ':') {
				MyChDrive (szDefDir[0]);
				MyChDir (&szDefDir[1]);
			} else	
				MyChDir (szDefDir);
		}		
		// Rip out terminating zero which adds parsed filename.  You
		// think this is strange?  The original FindExecutable also
		// appends the arguments past the first zero.
		szTemp[lstrlen (szTemp)] = ' ';

		// BTW, we know there are no params because they are only valid
		// for executable files and those are passed directly to the 
		// old ShellExecute.
		
		// Launch program
		hInstRC = WinExec (szTemp, fsShowCmd);
	} else
		hInstRC = (lpfnSHELLEXECUTE) (hWnd, lpszOp, lpszFile, 
		                              lpszParams, lpszDir, fsShowCmd);
	return hInstRC;
}
//===================================================================
// Local procedures
//===================================================================
//-------------------------------------------------------------------
// GetEditString - Gets the editor string from the INI file.
//-------------------------------------------------------------------
void GetEditString (LPSTR lpszEditor, INT sMax) {
		GetPrivateProfileString ("DefEdit", PRO_EDITOR, DEFEDITSTRING,
		                         lpszEditor, sMax,
		                         "DEFEDIT.INI");
	return;
}	
//-------------------------------------------------------------------
// ParseEditCmd - Parses the editor string.
//-------------------------------------------------------------------
void ParseEditCmd (LPSTR lpszEditor, LPSTR lpszFileName, 
                   LPSTR lpszDir, LPSTR lpszResult) {
	LPSTR lpszName, lpszExt, lpszTemp, lpszOut;
	BOOL fParsed = FALSE;
	char ch;

	// Add fix for File Man who uses fancy dir syntax 
	if ((lpszDir) && (*lpszDir)) {
		lpszTemp = lpszFileName;
		while ((*lpszTemp != '\0') && (*lpszTemp != '\\'))
			lpszTemp++;
		// If backslash found, path info included in filename. Scan
		// for . indicating current dir.  The ..'s should be handled
		// by DOS when interpeting the path of the filename.
		if (*lpszTemp == '\\') {
			ch = '\\';
			lpszTemp = lpszFileName;
			lpszOut = szDefDir;
			while (*lpszTemp) {
				if ((*lpszTemp == '.') && (*(lpszTemp+1) == '\\') &&
				    (ch == '\\')) {
					lstrcpy (lpszOut, lpszDir);					
					if (*(lpszOut+lstrlen(lpszOut)-1) == '\\')
						lpszTemp++;
					lpszOut += lstrlen (lpszOut);
				} else
					*lpszOut++ = *lpszTemp;
				ch = *lpszTemp++;
			}			
			*lpszOut = '\0';
			
		// Else just join specified directory and filename.	
		} else {
			lstrcpy (szDefDir, lpszDir);
			if (szDefDir[lstrlen (szDefDir)-1] != '\\')
				lstrcat (szDefDir, "\\");
			lstrcat (szDefDir, lpszFileName);
		}				
		lpszFileName = szDefDir;
	}		
	// Sep name from path
	lpszName = lpszFileName + lstrlen (lpszFileName);
	while ((lpszName > lpszFileName) && (*lpszName != '\\'))
		lpszName--;
	if (*lpszName == '\\') 
		lpszName++;
	// Find extension if any
	lpszExt = lpszName;
	while ((*lpszExt != '\0') && (*lpszExt != '.'))
		lpszExt++;
	if (*lpszExt == '.') 
		lpszExt++;

	*lpszResult = '\0';
	lpszOut = lpszResult;
	while (*lpszEditor != '\0') {
		if (*lpszEditor == '^') {
			lpszEditor++;
			// See if path desired
			if (*lpszEditor == 'p') {
				lpszTemp = lpszFileName;
				while ((*lpszTemp != '\0') && (lpszTemp < lpszName-1)) 
					*lpszOut++ = *lpszTemp++;
				if (*(lpszTemp - 1) == ':')
					*lpszOut++ = '\\';
				*lpszOut++ = '\0';

			// See if filename only desired
			} else if (*lpszEditor == 'f') {
				lpszTemp = lpszName;
				while ((*lpszTemp != '\0') && (lpszTemp < lpszExt-1)) 
					*lpszOut++ = *lpszTemp++;
				*lpszOut++ = '\0';

			// See if extension desired
			} else if (*lpszEditor == 'e') {
				lstrcpy (lpszOut, lpszExt);

			// Must be default, provides path + filename - extension
			} else {
				lpszTemp = lpszFileName;
				while ((*lpszTemp != '\0') && (*lpszTemp != '.'))
					*lpszOut++ = *lpszTemp++;
				*lpszOut++ = '\0';
				lpszEditor--;
				}	
			// Update ptrs
			lpszEditor++;
			lpszOut = lpszResult + lstrlen (lpszResult);
			fParsed = TRUE;
		} else
			*lpszOut++ = *lpszEditor++;
	}
	*lpszOut = '\0';
	// If no special characters, append filenme onto end of editor.
	if (!fParsed) {
		lstrcat (lpszResult, " ");
		lstrcat (lpszResult, lpszFileName);
	} 
	return;
}	
//-------------------------------------------------------------------
// MyChDir - My version of a Change Directory function
//-------------------------------------------------------------------
int MyChDir (LPSTR lpszNewDir) {
	int rc;
	
	_asm {
		push	ds
		push	dx
		lds	dx,lpszNewDir
		mov	ah,3bh
		int	21h
		pop	dx
		pop	ds
		mov	rc,ax
	}
	return rc;
}		
//-------------------------------------------------------------------
// MyChDir - My version of a Change Drive function
//-------------------------------------------------------------------
void MyChDrive (char chNewDrive) {

	chNewDrive = (char)((chNewDrive & 0xdf) - 0x40);
	_asm {
		push	dx
		mov	dl,chNewDrive
		mov	ah,0eh
		int	21h
		pop	dx
	}
	return;
}		
