//===========================================================
// DEFEDIT - Provides a default editor for Windows.
// Copyright (C) 1995 Douglas Boling
//===========================================================
//-----------------------------------------------------------
// Include files
//-----------------------------------------------------------
#include <windows.h>
#include <commdlg.h>
#include <shellapi.h>

#include "stdlib.h"
#include "dos.h"
#include "stdio.h"
#include "direct.h"
#include "string.h"
#include "defedit.h"

typedef struct DOSERROR DOSERR;
//-----------------------------------------------------------
// Function prototypes
//-----------------------------------------------------------
//
// Message functions
//
LONG DoSizeMain (HWND, UINT, UINT, LONG);
LONG DoPaintMain (HWND, UINT, UINT, LONG);
LONG DoCommandMain (HWND, UINT, UINT, LONG);
LONG DoCloseMain (HWND, UINT, UINT, LONG);
LONG DoDestroyMain (HWND, UINT, UINT, LONG);
//
// Button functions
//
LONG DoMainCtlInstall (HWND, UINT, HWND, UINT);
LONG DoMainCtlSetEdit (HWND, UINT, HWND, UINT);
LONG DoMainCtlExit (HWND, UINT, HWND, UINT);
//------------------------------------------------------------
// Message dispatch table for MainWindowProc
//------------------------------------------------------------
struct decodeUINT MainMessages[] = {
	WM_SIZE, DoSizeMain,
	WM_PAINT, DoPaintMain,
	WM_COMMAND, DoCommandMain,
	WM_CLOSE, DoCloseMain, 
	WM_DESTROY, DoDestroyMain, 
};
//------------------------------------------------------------
// Command Message dispatch for MainWindowProc
//------------------------------------------------------------
struct decodeCMD MainCMDItems[] = {
	IDD_INSTALL, DoMainCtlInstall,
	IDD_SETEDIT, DoMainCtlSetEdit,
	IDCANCEL, DoMainCtlExit,
	IDD_EXIT, DoMainCtlExit,
};
//-----------------------------------------------------------
// Global data
//-----------------------------------------------------------
char szAppName[] = "DefEdit";            // Application name
char szIconName[] = "defeditIcon";       // Icon name
char szProfileName[] = "DEFEDIT.INI";    // INI file name

char szBatFile[128];

HANDLE hInst;
HWND hMain;
//============================================================
// WinMain -- entry point for this application from Windows.
//============================================================
INT APIENTRY WinMain(HANDLE hInstance, HANDLE hPrevInstance, 
                     LPSTR lpCmdLine, INT nCmdShow) {
	MSG	msg;
	INT	rc;

	hInst = hInstance;
	// If not first instance terminate.
   if (hPrevInstance)
		return 0;

	// Init Application
	if((rc = InitApp(hInstance)) != 0)
		return rc;

	// Initialize this instance
	if((rc = InitInstance(hInstance, lpCmdLine, nCmdShow)) != 0)
		return rc;

	// Application message loop
	while (GetMessage (&msg, NULL, 0, 0)) {
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}
	// Instance cleanup
	return TermInstance(hInstance, msg.wParam);
}
//-----------------------------------------------------------
// InitApp - Global initialization code for this application.
//-----------------------------------------------------------
INT InitApp(HANDLE hInstance) {
	WNDCLASS  wc;
	//
	// Register App Window class
	//
	wc.style = 0;                             // Window style
	wc.lpfnWndProc = MainWndProc;             // Callback function
	wc.cbClsExtra = 0;                        // Extra class data
	wc.cbWndExtra = DLGWINDOWEXTRA;           // Extra window data
	wc.hInstance = hInstance;                 // Owner handle
	wc.hIcon = LoadIcon(hInst, szIconName);   // Application icon
	wc.hCursor = LoadCursor(NULL, IDC_ARROW); // Default cursor
	wc.hbrBackground = GetStockObject(LTGRAY_BRUSH); // Background color 
	wc.lpszMenuName = 0;                      // Menu name
	wc.lpszClassName = szAppName;             // Window class name

	if (!RegisterClass(&wc))
		return 1;
 	return 0;
}

//-----------------------------------------------------------
// InitInstance - Instance initialization code for this app.
//-----------------------------------------------------------
INT InitInstance(HANDLE hInstance, LPSTR lpCmdLine, INT nCmdShow) {
	int	x,y;

	x  = GetPrivateProfileInt (szAppName, PRO_XPOS, 100, szProfileName);
	y  = GetPrivateProfileInt (szAppName, PRO_YPOS, 100, szProfileName);
	
   hMain = CreateDialog (hInstance, szAppName, 0, NULL);
	if(!hMain) return 0x10;           
	SetWindowPos (hMain, NULL, x, y, 0, 0, SWP_NOSIZE);

	//Show window only if not installed.
	ShowWindow(hMain, nCmdShow | SW_SHOW);
	UpdateWindow(hMain);              // force WM_PAINT message
	return 0;                         // return success flag
}

//------------------------------------------------------------
// TermInstance - Instance termination code for this app.
//------------------------------------------------------------
INT TermInstance(HANDLE hinstance, int sDefRC) {
	return sDefRC;
}
//============================================================
// Message handling procedures for MainWindow
//============================================================
//------------------------------------------------------------
// MainWndProc - Callback function for application window
//------------------------------------------------------------
LONG CALLBACK MainWndProc(HWND hWnd, UINT wMsg, UINT wParam, 
                          LONG lParam) {
	INT i;
	//
	// Search message list to see if we need to handle this
	// message.  If in list, call procedure.
	//
	for(i = 0; i < dim(MainMessages); i++) {
		if(wMsg == MainMessages[i].Code)
			return (*MainMessages[i].Fxn)(hWnd, wMsg, wParam, lParam);
	}
	return DefWindowProc(hWnd, wMsg, wParam, lParam);
}
//------------------------------------------------------------
// DoSizeMain - process WM_SIZE message for frame window.
//------------------------------------------------------------ 
LONG DoSizeMain (HWND hWnd, UINT wMsg, UINT wParam, LONG lParam) {
	INT rc;
	
	rc = ChkInstalled ();
	if (rc == INFO_INSTALLED) 
		SetDlgItemText (hWnd, IDD_INSTALL, "&Uninstall");
	else {
		SetDlgItemText (hWnd, IDD_INSTALL, "&Install");
		if (rc != INFO_NOTINSTALLED)
			EnableWindow (GetDlgItem (hWnd, IDD_INSTALL), FALSE);
	}
	return DefDlgProc(hWnd, wMsg, wParam, lParam);
}
//------------------------------------------------------------
// DoPaintMain - process WM_PAINT message for frame window.
//------------------------------------------------------------ 
LONG DoPaintMain (HWND hWnd, UINT wMsg, UINT wParam, LONG lParam) {
	HDC hdc;
	HPEN hDPen, hLPen, hOldPen;
	PAINTSTRUCT ps;
	RECT rectAbout;
	//
	// Draw the rectangle surrounding the dialog box text.
	//
	hdc = BeginPaint (hWnd, &ps);
	//Get text rectangle defined in RC file
	GetClientRect (GetDlgItem (hWnd, IDD_FRAME), &rectAbout);
	ClientToScreen (GetDlgItem (hWnd, IDD_FRAME), (LPPOINT)&rectAbout);
	ScreenToClient (hWnd, (LPPOINT)&rectAbout);
	rectAbout.left -= 2;
	rectAbout.top -= 2;
	rectAbout.right += rectAbout.left + 4;
	rectAbout.bottom += rectAbout.top + 4;
	hDPen = CreatePen (PS_SOLID, 1, GetSysColor (COLOR_BTNSHADOW));
	hLPen = CreatePen (PS_SOLID, 1, GetSysColor (COLOR_BTNHIGHLIGHT));

	//Start at bottom left, draw dark pen up and over.
	hOldPen = SelectObject (hdc, hDPen);
	MoveTo (hdc, rectAbout.left, rectAbout.bottom);
	LineTo (hdc, rectAbout.left, rectAbout.top);
	LineTo (hdc, rectAbout.right+1, rectAbout.top);
	//Start at bottom left, draw light pen over and up.
	SelectObject (hdc, hLPen);
	MoveTo (hdc, rectAbout.left+1, rectAbout.bottom);
	LineTo (hdc, rectAbout.right, rectAbout.bottom);
	LineTo (hdc, rectAbout.right, rectAbout.top);

	SelectObject (hdc, hOldPen);
	DeleteObject (hDPen);
	DeleteObject (hLPen);
	EndPaint (hWnd, &ps);
	return DefDlgProc(hWnd, wMsg, wParam, lParam);
}
//------------------------------------------------------------
// DoCommandMain - process WM_COMMAND message for frame window 
// by decoding the menubar item with the CMDItems[] array, 
// then running the corresponding function to process the command.
//------------------------------------------------------------ 
LONG DoCommandMain (HWND hWnd, UINT wMsg, UINT wParam, LONG lParam) {
	INT	i;
	UINT	idItem, wNotifyCode;
	HWND	hwndCtl;

	idItem = (UINT) wParam;                      // Parse Parameters
	hwndCtl = (HWND) LOWORD(lParam);
	wNotifyCode = (UINT) HIWORD(lParam);
	//
	// Call routine to handle control message
	//
	for(i = 0; i < dim(MainCMDItems); i++) {
		if(idItem == MainCMDItems[i].Code)
			return (*MainCMDItems[i].Fxn)(hWnd, idItem, hwndCtl, 
			                               wNotifyCode);
	}
	return DefWindowProc(hWnd, wMsg, wParam, lParam);
}
//------------------------------------------------------------
// DoCloseMain - process WM_CLOSE message for frame window.
//------------------------------------------------------------ 
LONG DoCloseMain (HWND hWnd, UINT wMsg, UINT wParam, LONG lParam) {

	DestroyWindow(hWnd);
	return 0;
}
//------------------------------------------------------------
// DoDestroyMain - process WM_DESTROY message for frame window.
//------------------------------------------------------------ 
LONG DoDestroyMain (HWND hWnd, UINT wMsg, UINT wParam, LONG lParam) {
   RECT	rect;

	//Save Window position
	if	(!IsIconic (hWnd)) {
		GetWindowRect (hWnd, &rect);
		MyWritePrivateProfileInt (szAppName, PRO_XPOS, rect.left,
		                          szProfileName);
		MyWritePrivateProfileInt (szAppName, PRO_YPOS, rect.top,
		                          szProfileName);
	}
	PostQuitMessage (0);
	return 0;
}
//============================================================
// Control handling procedures for MainWindow
//============================================================
//------------------------------------------------------------
// DoCtlInstall - Process Install button
//------------------------------------------------------------ 
LONG DoMainCtlInstall (HWND hWnd, UINT idItem, HWND hwndCtl, 
                    UINT wNotifyCode) {
	INT rc;
	BOOL fInstalling;
	char szCmd[128];
	char szParms[256];

	rc = ChkInstalled ();
	if (rc == INFO_INSTALLED) {
		fInstalling = FALSE;
		rc = InstallProg (FALSE);		
		if (rc)
			PrintError (hWnd, rc);
	} else if (rc == INFO_NOTINSTALLED) {
		fInstalling = TRUE;
		rc = InstallProg (TRUE);
		if (rc)
			PrintError (hWnd, rc);
	} else {
		SetDlgItemText (hWnd, IDD_INSTALL, "&Install");
		EnableWindow (GetDlgItem (hWnd, IDD_INSTALL), FALSE);
		rc = 1;
	}
	// Tell user we must restart windows to run the install BAT file.
	if (rc == 0) {
		if (MyDisplayDialog(hInst, "RestartBox", hWnd, 
   		             (DLGPROC) RestartDlgProc, 0)) {
			lstrcpy (szCmd, getenv ("COMSPEC"));
			lstrcpy (szParms, " /c ");
			lstrcat (szParms, szBatFile);
			if (!ExitWindowsExec (szCmd, szParms))
				rc = 1;
		} else
			rc = 1;
		if (rc)
			if (fInstalling)
				lstrcpy (szParms, "DefEdit Install not complete.\n\n"\
				         "To complete the install follow"\
				         " these steps:\n\n"\
			            "1. Exit Windows.\n"\
			            "2. Switch to the DefEdit directory.\n"\
			            "3. Run the file DEINST.BAT.\n"\
			            "4. Restart Windows.");
			else							
				lstrcpy (szParms, "DefEdit uninstall not complete.\n\n"\
				         "To complete the uninstall"\
				         " follow these steps:\n\n"\
			            "1. Exit Windows.\n"\
			            "2. Switch to the DefEdit directory.\n"\
			            "3. Run the file DEINST.BAT.\n"\
			            "4. Restart Windows.");
							
			MessageBox (hWnd, szParms, "DefEdit", 
			            MB_OK | MB_ICONINFORMATION);
	}
	return 0;
}
//------------------------------------------------------------
// DoCtlSetEdit - Process Set Editor button
//------------------------------------------------------------ 
LONG DoMainCtlSetEdit (HWND hWnd, UINT idItem, HWND hwndCtl, 
                       UINT wNotifyCode) {

	MyDisplayDialog(hInst, "SetEdit", hWnd, (WNDPROC) SetEditDlgProc, 0);
	return 0;
}
//------------------------------------------------------------
// DoMainCtlExit - Process Cancel button
//------------------------------------------------------------ 
LONG DoMainCtlExit (HWND hWnd, UINT idItem, HWND hwndCtl, 
                     UINT wNotifyCode) {
	SendMessage (hWnd, WM_CLOSE, 0, 0);
	return TRUE;
}
//============================================================
// SetEditDlgProc - Set Editor dialog box dialog procedure
//============================================================
BOOL CALLBACK SetEditDlgProc (HWND hWnd, UINT wMsg, UINT wParam, 
                              LONG lParam) {
	static char *szFilter[] = {"Programs", "*.EXE;*.COM;*.PIF;*.BAT", 
	                           "All Files (*.*)", "*.*", 
										"" };
	char szEditor[256];
	char szTemp[144];
                             
	if (wMsg == WM_INITDIALOG) {
		GetPrivateProfileString (szAppName, PRO_EDITOR, DEFEDITSTRING, 
		                         szEditor, sizeof (szEditor),
		                         szProfileName);
		SetDlgItemText (hWnd, IDD_EDITOR, szEditor);

	} else if (wMsg == WM_COMMAND) {
		switch (wParam) {
			case IDD_BROWSE:
				if (MyGetFilename (hWnd, szTemp, 
				                   sizeof (szTemp), *szFilter, 1))
				SetDlgItemText (hWnd, IDD_EDITOR, szTemp);
				return TRUE;
			
			case IDOK:
				GetDlgItemText (hWnd, IDD_EDITOR, szEditor, sizeof (szEditor));
				WritePrivateProfileString (szAppName, PRO_EDITOR, szEditor,
		                                 szProfileName);
				// If we are installed, notify new shell of change. We do
				// this by asking FindExec for an editor for a file with 
				// a strange extension.  I can't use the ChkInstalled 
				// routine because the new dll may be copied, but not loaded.
				if (FindExecutable ("dtoeusgt.Z$#", "", szTemp) == 33)
					FindExecutable (NULL, NULL, NULL);

			case IDCANCEL:
				EndDialog(hWnd, 0);
				return TRUE;
		}	
	}
	return FALSE;
}
//============================================================
// RestartDlgProc - Restart Windows dialog box dialog procedure
//============================================================
BOOL CALLBACK RestartDlgProc (HWND hWnd, UINT wMsg, UINT wParam, 
                              LONG lParam) {
	switch (wMsg) {
		case WM_COMMAND:
	   	switch (wParam) {
	   		case IDOK:
					EndDialog(hWnd, 1);
					break;
	   		case IDCANCEL:
					EndDialog(hWnd, 0);
					break;
			}	
			return TRUE;
	}
	return FALSE;
}
//------------------------------------------------------------
// GetShellPath - Returns the full path of SHELL.DLL
//------------------------------------------------------------ 
void GetShellPath (char *pszOut, INT sMax) {
	
	GetSystemDirectory (pszOut, sMax);
	if (*(pszOut+lstrlen (pszOut)) != '\\')
		lstrcat (pszOut, "\\");
	lstrcat (pszOut, "SHELL.DLL");
	return;		
}
//------------------------------------------------------------
// InstallProg - Install or remove program.
//------------------------------------------------------------ 
INT InstallProg (BOOL fInstall) {
	INT i, rc = 0;
	char szShellName[128];
	char szOShellName[128];
	char szDEDLLName[128];
	char szCmd[128];
	HFILE hFile;
	OFSTRUCT of;
	
	// Get full path of Shell DLL
	GetShellPath (szShellName, sizeof (szShellName));

	// Create full path of Old Shell DLL
	lstrcpy (szOShellName, szShellName);
	i = lstrlen (szOShellName);
	while (szOShellName[i--] != '\\');
	lstrcpy (&szOShellName[i+2], "OLDSHELL.DLL");

	// Create full path of DEDLL DLL
 	i = GetModuleFileName (hInst, szDEDLLName, sizeof (szDEDLLName));
	while (szDEDLLName[i--] != '\\');
	lstrcpy (&szDEDLLName[i+2], "DEDLL.DLL");

	// Create BAT file needed for install/uninstall
 	i = GetModuleFileName (hInst, szBatFile, sizeof (szBatFile));
	while (szBatFile[i--] != '\\');
	lstrcpy (&szBatFile[i+2], "DEINST.BAT");

	hFile = OpenFile (szBatFile, &of, OF_CREATE | OF_READWRITE |
	                  OF_SHARE_EXCLUSIVE);
	if (hFile == HFILE_ERROR)
		return ERR_DOSOPEN - of.nErrCode;

	lstrcpy (szCmd, "@echo off\r\n");
	_lwrite (hFile, szCmd, lstrlen (szCmd));

	if (fInstall) {
		lstrcpy (szCmd, "echo Installing DefEdit\r\n");
		_lwrite (hFile, szCmd, lstrlen (szCmd));

		// Create cmd to copy Shell to Old Shell
		lstrcpy (szCmd, "Copy ");
		lstrcat (szCmd, szShellName);
		lstrcat (szCmd, " ");
		lstrcat (szCmd, szOShellName);
		lstrcat (szCmd, "\r\n");
		_lwrite (hFile, szCmd, lstrlen (szCmd));

		// Add if exist statement to confirm copy
		lstrcpy (szCmd, "if not exist ");
		lstrcat (szCmd, szOShellName);
		lstrcat (szCmd, " goto l1\r\n");
		_lwrite (hFile, szCmd, lstrlen (szCmd));

		// Create cmd to copy DefEdit Shell to Shell
		lstrcpy (szCmd, "Copy ");
		lstrcat (szCmd, szDEDLLName);
		lstrcat (szCmd, " ");
		lstrcat (szCmd, szShellName);
		lstrcat (szCmd, "\r\n");
		_lwrite (hFile, szCmd, lstrlen (szCmd));

		lstrcpy (szCmd, ":l1\r\n");
		_lwrite (hFile, szCmd, lstrlen (szCmd));

	} else {
		lstrcpy (szCmd, "echo Removing DefEdit\r\n");
		_lwrite (hFile, szCmd, lstrlen (szCmd));

		// Create cmd to copy old Shell to Shell
		lstrcpy (szCmd, "Copy ");
		lstrcat (szCmd, szOShellName);
		lstrcat (szCmd, " ");
		lstrcat (szCmd, szShellName);
		lstrcat (szCmd, "\r\n");
		_lwrite (hFile, szCmd, lstrlen (szCmd));

		// Create cmd to delete old Shell file
		lstrcpy (szCmd, "Del ");
		lstrcat (szCmd, szOShellName);
		lstrcat (szCmd, "\r\n");
		_lwrite (hFile, szCmd, lstrlen (szCmd));
	}
	_lclose (hFile);
	return rc;		
}
//------------------------------------------------------------
// ChkInstalled - See if already installed
//------------------------------------------------------------ 
INT ChkInstalled (void) {
	INT i, rc;
	char szShellName[256];
	char szTemp[256];
	char szTemplate1[] =	"Windows 3.1 Shell API Library";
	char szTemplate2[] =	"New Shell DLL (c) 1995, Douglas Boling";

	GetShellPath (szShellName, sizeof (szShellName));
	rc = GetFileDescStr (szShellName, szTemp);
	if (rc == 0) {
		if (lstrcmp (szTemp, szTemplate1) == 0)
			rc = INFO_NOTINSTALLED;
		else if (lstrcmp (szTemp, szTemplate2) == 0) {

			// Replace shell name with old shell name.
			i = lstrlen (szShellName);
			while (szShellName[i--] != '\\');
			lstrcpy (&szShellName[i+2], "OLDSHELL.DLL");
			rc = GetFileDescStr (szShellName, szTemp);
			if (rc == 0)
				if (lstrcmp (szTemp, szTemplate1) == 0)
					rc = INFO_INSTALLED;
				else	
					rc = ERR_UNKSHELL;	
		} else
			rc = ERR_UNKSHELL;	
	}
	return rc;		
}
//------------------------------------------------------------
// GetFileDescStr - Returns the description string from a
// NE program file.
//------------------------------------------------------------ 
INT GetFileDescStr (char *szName, char *pszOut) {
	UINT usBytesRead;
	INT rc = 0;
	HLOCAL hLocal;
	PBYTE pLBuff;
	HFILE hFile;
	LONG lFilePtr;

	hFile = MyOpenFile (szName, OF_READ | OF_SHARE_DENY_NONE, &rc);
	if (rc)
		return rc;

	hLocal = LocalAlloc (LHND, 0x200);
	if (hLocal == 0) {
		_lclose (hFile);
		return ERR_OUTOFMEM;
	}	
	pLBuff = LocalLock (hLocal);

	usBytesRead = _lread (hFile, pLBuff, 0x40);
	//Check for "MZ" and check for New EXE header.
	if ((usBytesRead < 0x40) ||
	    (*(UINT *)pLBuff != 0x5a4d) ||
	    (*((UINT *)(pLBuff+0x18)) < 0x40)) {
		LocalUnlock (hLocal);
		LocalFree (hLocal);
		_lclose (hFile);
		return ERR_NOTNEFILE;
	} 
	//Read New EXE header
	lFilePtr = *((PLONG)(pLBuff+0x3C));
	_llseek (hFile, lFilePtr, SEEK_SET);
	usBytesRead = _lread (hFile, pLBuff, 0x200);

	//Check for NewEXE (NE) and Windows file.
	if ((*(UINT *)pLBuff == 0x454e) && (*(pLBuff+0x36) & 2)) {
		//Get description string.  It is in the 1st entry in nonresident
		// names table. This hack works because the first entry following
		// the description has an ordnal number of zero which
		// provides the terminating 0. The extra byte of seek skips 
		// the name length byte.
		_llseek (hFile, ((LONG)*((UINT *)(pLBuff+0x2C)) + 1), 
		         SEEK_SET);
		usBytesRead = _lread (hFile, pLBuff, 0x100);
		lstrcpy (pszOut, pLBuff);
	} else 
		rc = ERR_NOTNEFILE;
	
	_lclose (hFile);
	LocalUnlock (hLocal);
	LocalFree (hLocal);
	return rc;
}	
//============================================================  
// General Helper Routines 
//============================================================  
//-------------------------------------------------------------------
// MyOpenFile - File open wrapper
//-------------------------------------------------------------------
HFILE MyOpenFile (char *szName, INT sFlags, INT *pRC) {
	HFILE handle;
	OFSTRUCT of;	

	handle = OpenFile (szName, &of, sFlags);
	if (handle == HFILE_ERROR) {
		handle = 0;
		*pRC = ERR_DOSOPEN - of.nErrCode;	
	} else	
		*pRC = 0;	
	return handle;
}	
//------------------------------------------------------------
// MyCopyFile - Copy a file
//------------------------------------------------------------
#define COPYBUFFSIZE 0xf000	
INT MyCopyFile (char *szSrcFile, char *szDestFile) {
	
	int		rc = 0;
	HFILE		hSrc, hDest;
	HGLOBAL	hMem;
	UINT		usByteCnt;
	LPSTR		lpBuffer;
	DOSERR	doserr;
	
	// Open Source file
	hSrc = MyOpenFile (szSrcFile, OF_READ | OF_SHARE_DENY_NONE, &rc);
	if (rc)
		return rc;
	// Open Dest file
	hDest = MyOpenFile (szDestFile, OF_CREATE, &rc);
	if (rc) {
		_lclose (hSrc);
		return rc;
	}
	// Allocate memory for a copy buffer
	rc = 0;
	hMem = GlobalAlloc (GHND, (LONG) COPYBUFFSIZE);
	if (hMem != 0) {
		lpBuffer = GlobalLock (hMem);
		while (rc == 0) {
			usByteCnt = _lread (hSrc, lpBuffer, COPYBUFFSIZE);
			if (usByteCnt == 0xFFFF)
				rc = dosexterr (&doserr);
			else {
				usByteCnt = _lwrite (hDest, lpBuffer, usByteCnt);
				if (usByteCnt == 0xFFFF)
					rc = dosexterr (&doserr);
			}
			if (usByteCnt < COPYBUFFSIZE)
				break;
		}
		// Clean up
		GlobalUnlock (hMem);
		GlobalFree (hMem);
	} else
		rc = ERR_OUTOFMEM;
	_lclose (hSrc);
	_lclose (hDest);
	return rc;
}
//------------------------------------------------------------
// PrintError - Displays a message box with an error code
//------------------------------------------------------------ 
void PrintError (HWND hWnd, INT rc) {
	char szErrStr[80];
	char szTemp[12];
	
   rc = abs (rc);
	if (LoadString (hInst, rc, szErrStr, sizeof (szErrStr)) == 0) {
		itoa (rc, szTemp, 10);
		lstrcpy (szErrStr, "Error number: ");
		lstrcat (szErrStr, szTemp);
	}
	MessageBox (hWnd, szErrStr, szAppName, MB_OK | MB_ICONHAND);
	return;
}
//------------------------------------------------------------
// MyWritePrivateProfileInt - Writes an integer to the profile
//------------------------------------------------------------
BOOL MyWritePrivateProfileInt (char *szSec, char *szEntry, 
                               int Num, char *szProfile) {
	char	szStr[33];
	                           
	itoa (Num, szStr, 10);
	return WritePrivateProfileString (szSec, szEntry, szStr, 
	                                  szProfile);
}
//------------------------------------------------------------
// MyDisplayDialog - Display a dialog box
//------------------------------------------------------------ 
INT MyDisplayDialog (HINSTANCE hInstance, LPCSTR szDlgName,
                     HWND hWnd, WNDPROC lpDialogProc, 
                     LPARAM lParam) {
    WNDPROC lpDlgProcInst;
    INT		rc;

    lpDlgProcInst = MakeProcInstance(lpDialogProc, hInst);
    rc = DialogBoxParam (hInstance, szDlgName, hWnd, 
                         lpDlgProcInst, lParam);
    FreeProcInstance(lpDlgProcInst);
    return rc;                              
}
//------------------------------------------------------------
// MyGetFilename - Uses the common File Open dialog box to
// query the user for a filename
//------------------------------------------------------------
BOOL MyGetFilename (HWND hWnd, char *szFilename, INT sMaxLen, 
                    char *szFilter, INT sFilterIndex) {

   OPENFILENAME ofn;

   *szFilename = '\0';

	ofn.lStructSize = sizeof (OPENFILENAME);
	ofn.hwndOwner = hWnd;
	ofn.hInstance = hInst;
	ofn.lpstrFilter = szFilter;
	ofn.lpstrCustomFilter = 0;
	ofn.nMaxCustFilter = 0;
	ofn.nFilterIndex = sFilterIndex;
	ofn.lpstrFile = szFilename;
	ofn.nMaxFile = sMaxLen;
	ofn.lpstrFileTitle = 0;
	ofn.nMaxFileTitle = 0;
	ofn.lpstrInitialDir = 0;
	ofn.lpstrTitle = 0;
	ofn.Flags = OFN_FILEMUSTEXIST;
	ofn.nFileOffset = 0;
	ofn.nFileExtension = 0;
	ofn.lpstrDefExt = "EXE";
	ofn.lCustData = 0;
	ofn.lpfnHook = 0;
	ofn.lpTemplateName = 0;

 	return (GetOpenFileName (&ofn));
}
//------------------------------------------------------------
// MyYield - Yields control to other programs, but returns
// if Windows is idle.
//------------------------------------------------------------
BOOL MyYield (void) {
   MSG	msg;
   BOOL	bCont;
   
   bCont = TRUE;
	while (PeekMessage (&msg, NULL, 0, 0, PM_REMOVE)) {
	   if (msg.message == WM_QUIT)
	      bCont = FALSE;
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}
	return bCont;
}
